@@ -41,13 +41,14 @@ module Database.Relational.Query.Arrow (
 
   groupBy', key, key', set, bkey, rollup, cube, groupingSets,
 
-  orderBy, asc, desc,
+  orderBy', orderBy, asc, desc,
 
   partitionBy, over,
 
   assign,
 
   derivedUpdate', derivedUpdate,
+  derivedInsertValue', derivedInsertValue,
   derivedDelete', derivedDelete,
 
   QueryA,
@@ -58,7 +59,7 @@ module Database.Relational.Query.Arrow (
 
   Orderings, Window, Assignings,
 
-  AssignStatement, RestrictedStatement,
+  AssignStatement, Register, RestrictedStatement,
   ) where
 
 import Control.Category (Category)
@@ -74,9 +75,11 @@ import Database.Relational.Query hiding
    on, wheres, having, groupBy, placeholder,
    relation, relation', aggregateRelation, aggregateRelation', uniqueRelation',
    groupBy', key, key', set, bkey, rollup, cube, groupingSets,
-   orderBy, asc, desc, partitionBy, over,
-   derivedUpdate', derivedUpdate, derivedDelete', derivedDelete,
-   QuerySimple, QueryAggregate, QueryUnique, Window)
+   orderBy', orderBy, asc, desc, partitionBy, over,
+   derivedUpdate', derivedUpdate,
+   derivedInsertValue', derivedInsertValue,
+   derivedDelete', derivedDelete,
+   QuerySimple, QueryAggregate, QueryUnique, Window, Register)
 import qualified Database.Relational.Query as Monadic
 import Database.Relational.Query.Projection (ListProjection)
 import qualified Database.Relational.Query.Monad.Trans.Aggregating as Monadic
@@ -126,6 +129,9 @@ type Assignings r m = QueryA (Monadic.Assignings r m)
 -- | Arrow type corresponding to 'Monadic.AssignStatement'
 type AssignStatement r a = Assignings r Restrict (Projection Flat r) a
 
+-- | Arrow type corresponding to 'Monadic.Register'
+type Register r a = QueryA (Monadic.Register r) () a
+
 -- | Arrow type corresponding to 'Monadic.RestrictedStatement'
 type RestrictedStatement r a = QueryA Monadic.Restrict (Projection Flat r) a
 
@@ -364,6 +370,14 @@ cube = runAofM Monadic.cube
 groupingSets :: AggregatingSetList () a -> AggregateKey a
 groupingSets = runAofM Monadic.groupingSets
 
+-- | Same as 'Monadic.orderBy''.
+--   The result arrow is designed to be injected by local projections.
+orderBy' :: Monad m
+         => Order
+         -> Nulls
+         -> Orderings c m (Projection c t) ()
+orderBy' o n = queryA $ \p -> Monadic.orderBy' p o n
+
 -- | Same as 'Monadic.orderBy'.
 --   The result arrow is designed to be injected by local projections.
 orderBy :: Monad m
@@ -413,6 +427,16 @@ derivedUpdate' config = Monadic.derivedUpdate' config . runQueryA
 derivedUpdate :: TableDerivable r => AssignStatement r (PlaceHolders p) -> Update p
 derivedUpdate = Monadic.derivedUpdate . runQueryA
 
+-- | Same as 'Monadic.derivedInsertValue''.
+--   Make 'Insert' from register arrow using configuration.
+derivedInsertValue' :: TableDerivable r => Config -> Register r (PlaceHolders p) -> Insert p
+derivedInsertValue' config = Monadic.derivedInsertValue' config . ($ ()) . runQueryA
+
+-- | Same as 'Monadic.derivedInsertValue'.
+--   Make 'Insert' from register arrow.
+derivedInsertValue :: TableDerivable r => Register r (PlaceHolders p) -> Insert p
+derivedInsertValue = Monadic.derivedInsertValue . ($ ()) . runQueryA
+
 -- | Same as 'Monadic.derivedDelete''.
 --   Make 'Update' from restrict statement arrow using configuration.
 derivedDelete' :: TableDerivable r => Config -> RestrictedStatement r (PlaceHolders p) -> Delete p
