@@ -1,6 +1,6 @@
 -- |
 -- Module      : Database.Relational.Query.Monad.Register
--- Copyright   : 2015 Kei Hibino
+-- Copyright   : 2015-2017 Kei Hibino
 -- License     : BSD3
 --
 -- Maintainer  : ex8k.hibino@gmail.com
@@ -15,7 +15,9 @@ module Database.Relational.Query.Monad.Register (
   extract,
   ) where
 
-import Database.Relational.Query.Component (Config, Assignments)
+import Database.Relational.Query.Internal.BaseSQL (Assignment)
+import Database.Relational.Query.Internal.Config (Config)
+
 import Database.Relational.Query.Table (Table)
 import Database.Relational.Query.Monad.BaseType (ConfigureQuery, configureQuery)
 import Database.Relational.Query.Monad.Trans.Assigning (Assignings, extractAssignments)
@@ -25,5 +27,5 @@ import Database.Relational.Query.Monad.Trans.Assigning (Assignings, extractAssig
 type Register r = Assignings r ConfigureQuery
 
 -- | Run 'InsertStatement'.
-extract :: Assignings r ConfigureQuery a -> Config -> (a, Table r -> Assignments)
+extract :: Assignings r ConfigureQuery a -> Config -> (a, Table r -> [Assignment])
 extract = configureQuery . extractAssignments
