@@ -2,7 +2,7 @@
 
 -- |
 -- Module      : Database.Relational.Query.Monad.Trans.Config
--- Copyright   : 2013 Kei Hibino
+-- Copyright   : 2013-2017 Kei Hibino
 -- License     : BSD3
 --
 -- Maintainer  : ex8k.hibino@gmail.com
@@ -20,7 +20,7 @@ import Control.Monad.Trans.Class (lift)
 import Control.Monad.Trans.Reader (ReaderT, runReaderT, ask)
 import Control.Applicative (Applicative)
 
-import Database.Relational.Query.Component (Config)
+import Database.Relational.Query.Internal.Config (Config)
 
 
 -- | 'ReaderT' type to require query generate configuration.
