@@ -1,6 +1,6 @@
 -- |
 -- Module      : Database.Relational.Query.Monad.Trans.JoinState
--- Copyright   : 2013-2016 Kei Hibino
+-- Copyright   : 2013-2017 Kei Hibino
 -- License     : BSD3
 --
 -- Maintainer  : ex8k.hibino@gmail.com
@@ -9,6 +9,8 @@
 --
 -- This module provides state definition for
 -- "Database.Relational.Query.Monad.Trans.Join".
+--
+-- This is not public interface.
 module Database.Relational.Query.Monad.Trans.JoinState (
   -- * Join context
   JoinContext, primeJoinContext, updateProduct, joinProduct
@@ -17,8 +19,8 @@ module Database.Relational.Query.Monad.Trans.JoinState (
 import Prelude hiding (product)
 import Data.DList (toList)
 
-import qualified Database.Relational.Query.Sub as Product
-import Database.Relational.Query.Sub (ProductBuilder, JoinProduct)
+import Database.Relational.Query.Internal.Sub (ProductBuilder, JoinProduct)
+import qualified Database.Relational.Query.Internal.Sub as Product
 
 
 -- | JoinContext type for QueryJoin.
