@@ -11,23 +11,24 @@
 -- Contains normal interfaces.
 module Database.Relational.Query.Pi (
   -- * Projection path
-  Pi, pfmap, pap, (<.>), (<?.>), (<?.?>),
+  Pi, pfmap, pap, pzero, (<.>), (<?.>), (<?.?>),
 
   id', fst', snd'
   ) where
 
+import qualified Control.Category as Category
 import Database.Record
   (PersistableWidth, persistableWidth, PersistableRecordWidth)
 import Database.Record.Persistable
   (runPersistableRecordWidth)
 
 import Database.Relational.Query.Pi.Unsafe
-  (Pi, pfmap, pap, (<.>), (<?.>), (<?.?>), definePi)
+  (Pi, pfmap, pap, pzero, (<.>), (<?.>), (<?.?>), definePi)
 
 
 -- | Identity projection path.
-id' :: PersistableWidth a => Pi a a
-id' =  definePi 0
+id' :: Pi a a
+id' = Category.id
 
 -- | Projection path for fst of tuple.
 fst' :: PersistableWidth a => Pi (a, b) a -- ^ Projection path of fst.
