@@ -14,8 +14,8 @@
 -- This module defines operators on various polymorphic projections.
 module Database.Relational.Query.Projectable (
   -- * Projectable from SQL strings
-  SqlProjectable (unsafeProjectSqlTerms'), unsafeProjectSql',
-  unsafeProjectSqlTerms, unsafeProjectSql,
+  SqlProjectable (unsafeProjectSqlTerms), unsafeProjectSql',
+  unsafeProjectSql,
 
   -- * Projections of values
   value,
@@ -25,7 +25,7 @@ module Database.Relational.Query.Projectable (
 
   -- * Placeholders
   PlaceHolders, unsafeAddPlaceHolders, unsafePlaceHolders,
-  placeholder', placeholder, unitPlaceHolder,
+  pwPlaceholder, placeholder', placeholder, unitPlaceHolder, unitPH,
 
   -- * Projectable into SQL strings
   ProjectableShowSql (unsafeShowSql'), unsafeShowSql,
@@ -59,32 +59,29 @@ module Database.Relational.Query.Projectable (
 
   -- * 'Maybe' type projecitoins
   ProjectableMaybe (just, flattenMaybe),
-
-  -- * ProjectableFunctor and ProjectableApplicative
-  ProjectableFunctor (..), ProjectableApplicative (..), ipfmap
   ) where
 
 import Prelude hiding (pi)
 
 import Data.String (IsString)
-import Control.Applicative ((<$>))
 
 import Language.SQL.Keyword (Keyword)
 import qualified Language.SQL.Keyword as SQL
 
 import Database.Record
-  (PersistableWidth, persistableWidth, PersistableRecordWidth, derivedWidth,
+  (PersistableWidth, persistableWidth, PersistableRecordWidth,
    HasColumnConstraint, NotNull)
 import Database.Record.Persistable (runPersistableRecordWidth)
 
 import Database.Relational.Query.Internal.SQL (StringSQL, stringSQL, showStringSQL)
 import qualified Database.Relational.Query.Internal.Sub as Internal
 
+import Database.Relational.Query.ProjectableClass
+  (ProjectableFunctor (..), ProjectableApplicative (..), )
 import Database.Relational.Query.Context (Flat, Aggregated, Exists, OverWindow)
-import Database.Relational.Query.Pure
-  (ShowConstantTermsSQL, showConstantTermsSQL', ProductConstructor (..))
-import Database.Relational.Query.Pi (Pi)
-import qualified Database.Relational.Query.Pi as Pi
+import Database.Relational.Query.TupleInstances ()
+import Database.Relational.Query.ProjectableClass
+  (ShowConstantTermsSQL, showConstantTermsSQL, )
 import Database.Relational.Query.Projection
   (Projection, ListProjection)
 import qualified Database.Relational.Query.Projection as Projection
@@ -93,26 +90,20 @@ import qualified Database.Relational.Query.Projection as Projection
 -- | Interface to project SQL terms unsafely.
 class SqlProjectable p where
   -- | Unsafely project from SQL expression terms.
-  unsafeProjectSqlTerms' :: [StringSQL] -- ^ SQL expression strings
-                         -> p t         -- ^ Result projection object
-
--- | Unsafely project from SQL strings. String interface of 'unsafeProjectSqlTerms''.
-unsafeProjectSqlTerms :: SqlProjectable p
-                      => [String] -- ^ SQL expression strings
-                      -> p t      -- ^ Result projection object
-unsafeProjectSqlTerms =  unsafeProjectSqlTerms' . map stringSQL
+  unsafeProjectSqlTerms :: [StringSQL] -- ^ SQL expression strings
+                        -> p t         -- ^ Result projection object
 
 -- | Unsafely make 'Projection' from SQL terms.
 instance SqlProjectable (Projection Flat) where
-  unsafeProjectSqlTerms' = Projection.unsafeFromSqlTerms
+  unsafeProjectSqlTerms = Projection.unsafeFromSqlTerms
 
 -- | Unsafely make 'Projection' from SQL terms.
 instance SqlProjectable (Projection Aggregated) where
-  unsafeProjectSqlTerms' = Projection.unsafeFromSqlTerms
+  unsafeProjectSqlTerms = Projection.unsafeFromSqlTerms
 
 -- | Unsafely make 'Projection' from SQL terms.
 instance SqlProjectable (Projection OverWindow) where
-  unsafeProjectSqlTerms' = Projection.unsafeFromSqlTerms
+  unsafeProjectSqlTerms = Projection.unsafeFromSqlTerms
 
 class SqlProjectable p => OperatorProjectable p
 instance OperatorProjectable (Projection Flat)
@@ -120,7 +111,7 @@ instance OperatorProjectable (Projection Aggregated)
 
 -- | Unsafely Project single SQL term.
 unsafeProjectSql' :: SqlProjectable p => StringSQL -> p t
-unsafeProjectSql' =  unsafeProjectSqlTerms' . (:[])
+unsafeProjectSql' =  unsafeProjectSqlTerms . (:[])
 
 -- | Unsafely Project single SQL string. String interface of 'unsafeProjectSql''.
 unsafeProjectSql :: SqlProjectable p => String -> p t
@@ -132,7 +123,7 @@ nothing :: (OperatorProjectable (Projection c), SqlProjectable (Projection c), P
 nothing = proxyWidth persistableWidth
   where
     proxyWidth :: SqlProjectable (Projection c) => PersistableRecordWidth a -> Projection c (Maybe a)
-    proxyWidth w = unsafeProjectSqlTerms' $ replicate (runPersistableRecordWidth w) SQL.NULL
+    proxyWidth w = unsafeProjectSqlTerms $ replicate (runPersistableRecordWidth w) SQL.NULL
 
 {-# DEPRECATED unsafeValueNull "Use `nothing' instead of this." #-}
 -- | Deprecated. Polymorphic projection of SQL null value.
@@ -142,7 +133,7 @@ unsafeValueNull = nothing
 
 -- | Generate polymorphic projection of SQL constant values from Haskell value.
 value :: (ShowConstantTermsSQL t, OperatorProjectable p) => t -> p t
-value =  unsafeProjectSqlTerms' . showConstantTermsSQL'
+value = unsafeProjectSqlTerms . showConstantTermsSQL
 
 -- | Polymorphic proejction of SQL true value.
 valueTrue  :: (OperatorProjectable p, ProjectableMaybe p) => p (Maybe Bool)
@@ -474,23 +465,31 @@ unsafePlaceHolders =  PlaceHolders
 
 -- | No placeholder semantics
 unitPlaceHolder :: PlaceHolders ()
-unitPlaceHolder =  unsafePlaceHolders
+unitPlaceHolder = unsafePlaceHolders
+
+-- | No placeholder semantics. Same as `unitPlaceHolder`
+unitPH :: PlaceHolders ()
+unitPH = unitPlaceHolder
 
 -- | Unsafely cast placeholder parameter type.
 unsafeCastPlaceHolders :: PlaceHolders a -> PlaceHolders b
 unsafeCastPlaceHolders PlaceHolders = PlaceHolders
 
-unsafeProjectPlaceHolder' :: (PersistableWidth r, SqlProjectable p)
-                               => (PersistableRecordWidth r, p r)
-unsafeProjectPlaceHolder' =  unsafeProjectSqlTerms' . (`replicate` "?") <$> derivedWidth
-
-unsafeProjectPlaceHolder :: (PersistableWidth r, SqlProjectable p)
-                               => p r
-unsafeProjectPlaceHolder =  snd unsafeProjectPlaceHolder'
+-- | Provide scoped placeholder from width and return its parameter object.
+pwPlaceholder :: SqlProjectable p
+              => PersistableRecordWidth a
+              -> (p a -> b)
+              -> (PlaceHolders a, b)
+pwPlaceholder pw f = (PlaceHolders, f $ projectPlaceHolder pw)
+  where
+    projectPlaceHolder :: SqlProjectable p
+                       => PersistableRecordWidth a
+                       -> p a
+    projectPlaceHolder = unsafeProjectSqlTerms . (`replicate` "?") . runPersistableRecordWidth
 
 -- | Provide scoped placeholder and return its parameter object.
 placeholder' :: (PersistableWidth t, SqlProjectable p) => (p t -> a) ->  (PlaceHolders t, a)
-placeholder' f = (PlaceHolders, f unsafeProjectPlaceHolder)
+placeholder' = pwPlaceholder persistableWidth
 
 -- | Provide scoped placeholder and return its parameter object. Monadic version.
 placeholder :: (PersistableWidth t, SqlProjectable p, Monad m) => (p t -> m a) -> m (PlaceHolders t, a)
@@ -535,21 +534,6 @@ instance ProjectableIdZip PlaceHolders where
   leftId  = unsafeCastPlaceHolders
   rightId = unsafeCastPlaceHolders
 
--- | Weaken functor on projections.
-class ProjectableFunctor p where
-  -- | Method like 'fmap'.
-  (|$|) :: ProductConstructor (a -> b) => (a -> b) -> p a -> p b
-
--- | Same as '|$|' other than using inferred record constructor.
-ipfmap :: (ProjectableFunctor p, ProductConstructor (a -> b))
-       => p a -> p b
-ipfmap =  (|$|) productConstructor
-
--- | Weaken applicative functor on projections.
-class ProjectableFunctor p => ProjectableApplicative p where
-  -- | Method like '<*>'.
-  (|*|) :: p (a -> b) -> p a -> p b
-
 -- | Compose seed of record type 'PlaceHolders'.
 instance ProjectableFunctor PlaceHolders where
   _ |$| PlaceHolders = PlaceHolders
@@ -558,26 +542,9 @@ instance ProjectableFunctor PlaceHolders where
 instance ProjectableApplicative PlaceHolders where
   pf |*| pa = unsafeCastPlaceHolders (pf >< pa)
 
--- | Compose seed of record type 'Projection'.
-instance ProjectableFunctor (Projection c) where
-  (|$|) = Projection.pfmap
-
--- | Compose record type 'Projection' using applicative style.
-instance ProjectableApplicative (Projection c) where
-  (|*|) = Projection.pap
-
--- | Compose seed of projection path 'Pi' which has record result type.
-instance ProjectableFunctor (Pi a) where
-  (|$|) = Pi.pfmap
-
--- | Compose projection path 'Pi' which has record result type using applicative style.
-instance ProjectableApplicative (Pi a) where
-  (|*|) = Pi.pap
-
 infixl 7 .*., ./., ?*?, ?/?
 infixl 6 .+., .-., ?+?, ?-?
 infixl 5 .||., ?||?
-infixl 4 |$|, |*|
 infix  4 .=., .<>., .>., .>=., .<., .<=., `in'`, `like`, `likeMaybe`, `like'`, `likeMaybe'`
 infixr 3 `and'`
 infixr 2 `or'`
