@@ -1,3 +1,4 @@
+{-# OPTIONS_GHC -fno-warn-orphans #-}
 {-# LANGUAGE FlexibleInstances #-}
 
 -- |
@@ -11,16 +12,10 @@
 --
 -- This module defines interfaces between haskell pure values
 -- and query internal projection values.
-module Database.Relational.Query.Pure (
+module Database.Relational.Query.Pure () where
 
-  -- * Interface to specify record constructors.
-  ProductConstructor (..),
-
-  -- * Constant SQL Terms
-  ShowConstantTermsSQL (..), showConstantTermsSQL
-  ) where
-
-import Data.Monoid (mconcat)
+import Control.Applicative (pure)
+import Data.Monoid ((<>))
 import Data.Int (Int8, Int16, Int32, Int64)
 import Data.ByteString (ByteString)
 import qualified Data.ByteString.Lazy as LB
@@ -32,6 +27,7 @@ import qualified Data.Text.Lazy.Encoding as LT
 import Text.Printf (PrintfArg, printf)
 import Data.Time (FormatTime, Day, TimeOfDay, LocalTime, UTCTime, ZonedTime, formatTime)
 import Data.Time.Locale.Compat (defaultTimeLocale)
+import Data.DList (DList, fromList)
 
 import Language.SQL.Keyword (Keyword (..))
 import Database.Record
@@ -39,25 +35,14 @@ import Database.Record
 import Database.Record.Persistable
   (runPersistableRecordWidth)
 
-import Database.Relational.Query.Internal.SQL (StringSQL, stringSQL, showStringSQL)
+import Database.Relational.Query.Internal.SQL (StringSQL, stringSQL)
+
+import Database.Relational.Query.ProjectableClass (ShowConstantTermsSQL (..))
 
 
--- | Specify tuple like record constructors which are allowed to define 'ProjectableFunctor'.
-class ProductConstructor r where
-  -- | The constructor which has type 'r'.
-  productConstructor :: r
-
--- | ProductConstructor instance of pair.
-instance ProductConstructor (a -> b -> (a, b)) where
-  productConstructor = (,)
-
-
--- | Constant integral SQL expression.
-intExprSQL :: (Show a, Integral a) => a -> StringSQL
-intExprSQL =  stringSQL . show
-
-intTermsSQL :: (Show a, Integral a) => a -> [StringSQL]
-intTermsSQL =  (:[]) . intExprSQL
+-- | Constant integral SQL terms.
+intTermsSQL :: (Show a, Integral a) => a -> DList StringSQL
+intTermsSQL =  pure . stringSQL . show
 
 -- | Escape 'String' for constant SQL string expression.
 escapeStringToSqlExpr :: String -> String
@@ -70,16 +55,11 @@ escapeStringToSqlExpr =  rec  where
 stringExprSQL :: String -> StringSQL
 stringExprSQL =  stringSQL . ('\'':) . (++ "'") . escapeStringToSqlExpr
 
-stringTermsSQL :: String -> [StringSQL]
-stringTermsSQL =  (:[]) . stringExprSQL
+stringTermsSQL :: String -> DList StringSQL
+stringTermsSQL = pure . stringExprSQL
 
--- | Interface for constant SQL term list.
-class ShowConstantTermsSQL a where
-  showConstantTermsSQL' :: a -> [StringSQL]
-
--- | String interface of 'showConstantTermsSQL''.
-showConstantTermsSQL :: ShowConstantTermsSQL a => a -> [String]
-showConstantTermsSQL =  map showStringSQL . showConstantTermsSQL'
+-- | Constant SQL terms of '()'.
+instance ShowConstantTermsSQL ()
 
 -- | Constant SQL terms of 'Int8'.
 instance ShowConstantTermsSQL Int8 where
@@ -128,12 +108,12 @@ instance ShowConstantTermsSQL Char where
 
 -- | Constant SQL terms of 'Bool'.
 instance ShowConstantTermsSQL Bool where
-  showConstantTermsSQL' = (:[]) . stringSQL . d  where
+  showConstantTermsSQL' = pure . stringSQL . d  where
     d True  = "(0=0)"
     d False = "(0=1)"
 
-floatTerms :: (PrintfArg a, Ord a, Num a)=> a -> [StringSQL]
-floatTerms f = (:[]) . stringSQL $ printf fmt f  where
+floatTerms :: (PrintfArg a, Ord a, Num a)=> a -> DList StringSQL
+floatTerms f = pure . stringSQL $ printf fmt f  where
   fmt
     | f >= 0    = "%f"
     | otherwise = "(%f)"
@@ -146,9 +126,8 @@ instance ShowConstantTermsSQL Float where
 instance ShowConstantTermsSQL Double where
   showConstantTermsSQL' = floatTerms
 
-constantTimeTerms :: FormatTime t => Keyword -> String -> t -> [StringSQL]
-constantTimeTerms kw fmt t = [mconcat [kw,
-                                       stringExprSQL $ formatTime defaultTimeLocale fmt t]]
+constantTimeTerms :: FormatTime t => Keyword -> String -> t -> DList StringSQL
+constantTimeTerms kw fmt t = pure $ kw <> stringExprSQL (formatTime defaultTimeLocale fmt t)
 
 -- | Constant SQL terms of 'Day'.
 instance ShowConstantTermsSQL Day where
@@ -172,17 +151,12 @@ instance ShowConstantTermsSQL ZonedTime where
 instance ShowConstantTermsSQL UTCTime where
   showConstantTermsSQL' = constantTimeTerms TIMESTAMPTZ "%Y-%m-%d %H:%M:%S%z"
 
-showMaybeTerms :: ShowConstantTermsSQL a => PersistableRecordWidth a -> Maybe a -> [StringSQL]
+showMaybeTerms :: ShowConstantTermsSQL a => PersistableRecordWidth a -> Maybe a -> DList StringSQL
 showMaybeTerms wa = d  where
   d (Just a) = showConstantTermsSQL' a
-  d Nothing  = replicate (runPersistableRecordWidth wa) $ stringSQL "NULL"
+  d Nothing  = fromList . replicate (runPersistableRecordWidth wa) $ stringSQL "NULL"
 
 -- | Constant SQL terms of 'Maybe' type. Width inference is required.
 instance (PersistableWidth a, ShowConstantTermsSQL a)
          => ShowConstantTermsSQL (Maybe a) where
   showConstantTermsSQL' = showMaybeTerms persistableWidth
-
--- | Constant SQL terms of '(a, b)' type.
-instance (ShowConstantTermsSQL a, ShowConstantTermsSQL b)
-         => ShowConstantTermsSQL (a, b) where
-  showConstantTermsSQL' (a, b) = showConstantTermsSQL' a ++ showConstantTermsSQL' b
