@@ -70,7 +70,8 @@ import qualified Database.Relational.Query.Internal.UntypedTable as UntypedTable
 
 import Database.Relational.Query.Table (Table)
 import qualified Database.Relational.Query.Table as Table
-import Database.Relational.Query.Pure (showConstantTermsSQL')
+import Database.Relational.Query.ProjectableClass (showConstantTermsSQL)
+import Database.Relational.Query.Pure ()
 
 
 showsSetOp' :: SetOp -> StringSQL
@@ -302,7 +303,7 @@ showsQueryProduct =  rec  where
     [urec left',
      joinType (Internal.nodeAttr left') (Internal.nodeAttr right'), JOIN,
      urec right',
-     ON, foldr1 SQL.and $ ps ++ concat [ showConstantTermsSQL' True | null ps ] ]
+     ON, foldr1 SQL.and $ ps ++ concat [ showConstantTermsSQL True | null ps ] ]
     where ps = [ unsafeProjectionStringSql p | p <- rs ]
 
 -- | Shows join product of query.
