@@ -17,9 +17,10 @@ module Database.Relational.Query.Type (
   relationalQuerySQL,
 
   -- * Typed update statement
-  KeyUpdate (..), unsafeTypedKeyUpdate, typedKeyUpdate, typedKeyUpdateTable,
+  KeyUpdate (..), unsafeTypedKeyUpdate, typedKeyUpdate, typedKeyUpdateTable, derivedKeyUpdate,
   Update (..), unsafeTypedUpdate, typedUpdate', typedUpdate, derivedUpdate', derivedUpdate,
-  typedUpdateAllColumn, restrictedUpdateAllColumn, restrictedUpdateTableAllColumn,
+  typedUpdateAllColumn, derivedUpdateAllColumn', derivedUpdateAllColumn,
+  restrictedUpdateAllColumn, restrictedUpdateTableAllColumn,
 
   updateSQL,
 
@@ -44,7 +45,7 @@ import Data.Monoid ((<>))
 
 import Database.Record (PersistableWidth)
 
-import Database.Relational.Query.Internal.Config (Config (chunksInsertSize), defaultConfig)
+import Database.Relational.Query.Internal.Config (Config, defaultConfig)
 import Database.Relational.Query.Internal.SQL (showStringSQL)
 
 import Database.Relational.Query.Monad.BaseType (Relation, sqlFromRelationWith)
@@ -54,12 +55,12 @@ import Database.Relational.Query.Monad.Register (Register)
 import Database.Relational.Query.Relation (tableOf)
 import Database.Relational.Query.Effect
   (Restriction, restriction', UpdateTarget, updateTarget', liftTargetAllColumn', InsertTarget, insertTarget',
-   sqlWhereFromRestriction, sqlFromUpdateTarget, sqlFromInsertTarget)
+   sqlWhereFromRestriction, sqlFromUpdateTarget, piRegister, sqlChunkFromInsertTarget, sqlFromInsertTarget)
 import Database.Relational.Query.Pi (Pi)
 import Database.Relational.Query.Table (Table, TableDerivable, derivedTable)
 import Database.Relational.Query.Projectable (PlaceHolders)
 import Database.Relational.Query.SQL
-  (QuerySuffix, showsQuerySuffix, insertPrefixSQL, insertSQL, insertSizedChunkSQL,
+  (QuerySuffix, showsQuerySuffix, insertPrefixSQL,
    updateOtherThanKeySQL, updatePrefixSQL, deletePrefixSQL)
 
 
@@ -107,6 +108,13 @@ typedKeyUpdate tbl key = unsafeTypedKeyUpdate key $ updateOtherThanKeySQL tbl ke
 typedKeyUpdateTable :: TableDerivable r => Relation () r -> Pi r p -> KeyUpdate p r
 typedKeyUpdateTable =  typedKeyUpdate . tableOf
 
+-- derivedKeyUpdate'
+-- Config parameter is not yet required for KeyUpdate.
+
+-- | Make typed 'KeyUpdate' from derived table and key columns selector 'Pi'.
+derivedKeyUpdate :: TableDerivable r => Pi r p -> KeyUpdate p r
+derivedKeyUpdate = typedKeyUpdate derivedTable
+
 -- | Show update SQL string
 instance Show (KeyUpdate p a) where
   show = untypeKeyUpdate
@@ -144,6 +152,15 @@ derivedUpdate :: TableDerivable r => AssignStatement r (PlaceHolders p) -> Updat
 derivedUpdate = derivedUpdate' defaultConfig
 
 
+-- | Make typed 'Update' from 'Config', 'Table' and 'Restriction'.
+--   Update target is all column.
+typedUpdateAllColumn' :: PersistableWidth r
+                      => Config
+                      -> Table r
+                      -> Restriction p r
+                      -> Update (r, p)
+typedUpdateAllColumn' config tbl r = typedUpdate' config tbl $ liftTargetAllColumn' r
+
 -- | Make typed 'Update' from 'Table' and 'Restriction'.
 --   Update target is all column.
 typedUpdateAllColumn :: PersistableWidth r
@@ -152,6 +169,22 @@ typedUpdateAllColumn :: PersistableWidth r
                      -> Update (r, p)
 typedUpdateAllColumn tbl r = typedUpdate tbl $ liftTargetAllColumn' r
 
+-- | Make typed 'Update' from 'Config', derived table and 'AssignStatement'.
+--   Update target is all column.
+derivedUpdateAllColumn' :: (PersistableWidth r, TableDerivable r)
+                        => Config
+                        -> RestrictedStatement r (PlaceHolders p)
+                        -> Update (r, p)
+derivedUpdateAllColumn' config = typedUpdateAllColumn' config derivedTable .restriction'
+
+-- | Make typed 'Update' from 'defaultConfig', derived table and 'AssignStatement'.
+--   Update target is all column.
+derivedUpdateAllColumn :: (PersistableWidth r, TableDerivable r)
+                       => RestrictedStatement r (PlaceHolders p)
+                       -> Update (r, p)
+derivedUpdateAllColumn = derivedUpdateAllColumn' defaultConfig
+
+{-# DEPRECATED restrictedUpdateAllColumn "Use derivedUpdateAllColumn or typedUpdateAllColumn instead of this." #-}
 -- | Directly make typed 'Update' from 'Table' and 'Restrict' monad context.
 --   Update target is all column.
 restrictedUpdateAllColumn :: PersistableWidth r
@@ -160,6 +193,7 @@ restrictedUpdateAllColumn :: PersistableWidth r
                            -> Update (r, p)
 restrictedUpdateAllColumn tbl = typedUpdateAllColumn tbl . restriction'
 
+{-# DEPRECATED restrictedUpdateTableAllColumn "Use derivedUpdateAllColumn or typedUpdateAllColumn instead of this." #-}
 -- | Directly make typed 'Update' from 'Table' and 'Restrict' monad context.
 --   Update target is all column.
 restrictedUpdateTableAllColumn :: (PersistableWidth r, TableDerivable r)
@@ -197,22 +231,26 @@ unsafeTypedInsert :: String -> Insert a
 unsafeTypedInsert s = Insert s Nothing
 
 -- | Make typed 'Insert' from 'Table' and columns selector 'Pi' with configuration parameter.
-typedInsert' :: Config -> Table r -> Pi r r' -> Insert r'
-typedInsert' config tbl pi' = unsafeTypedInsert' (insertSQL pi' tbl) ci n  where
-  (ci, n) = insertSizedChunkSQL pi' tbl $ chunksInsertSize config
+typedInsert' :: PersistableWidth r => Config -> Table r -> Pi r r' -> Insert r'
+typedInsert' config tbl =
+  typedInsertValue' config tbl . insertTarget' . piRegister
 
 -- | Make typed 'Insert' from 'Table' and columns selector 'Pi'.
-typedInsert :: Table r -> Pi r r' -> Insert r'
+typedInsert :: PersistableWidth r => Table r -> Pi r r' -> Insert r'
 typedInsert =  typedInsert' defaultConfig
 
 -- | Table type inferred 'Insert'.
-derivedInsert :: TableDerivable r => Pi r r' -> Insert r'
-derivedInsert =  typedInsert derivedTable
+derivedInsert :: (PersistableWidth r, TableDerivable r) => Pi r r' -> Insert r'
+derivedInsert = typedInsert derivedTable
 
 -- | Make typed 'Insert' from 'Config', 'Table' and monadic builded 'InsertTarget' object.
 typedInsertValue' :: Config -> Table r -> InsertTarget p r -> Insert p
-typedInsertValue' config tbl  =
-  unsafeTypedInsert . showStringSQL . sqlFromInsertTarget config tbl
+typedInsertValue' config tbl it =
+    unsafeTypedInsert'
+    (showStringSQL $ sqlFromInsertTarget config tbl it)
+    (showStringSQL ci) n
+  where
+    (ci, n) = sqlChunkFromInsertTarget config tbl it
 
 -- | Make typed 'Insert' from 'Table' and monadic builded 'InsertTarget' object.
 typedInsertValue :: Table r -> InsertTarget p r -> Insert p
