@@ -2,9 +2,11 @@
 {-# LANGUAGE TemplateHaskell #-}
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
+{-# LANGUAGE DeriveGeneric #-}
 
 module Model where
 
+import GHC.Generics (Generic)
 import Data.Int (Int32, Int64)
 
 import Database.Relational.Query (defaultConfig)
@@ -15,14 +17,14 @@ $(defineTable defaultConfig "TEST" "set_a"
   [ ("int_a0" ,    [t| Int32  |])
   , ("str_a1" ,    [t| String |])
   , ("str_a2" ,    [t| String |]) ]
-  [] [0] $ Just 0)
+  [''Generic] [0] $ Just 0)
 
 
 $(defineTable defaultConfig "TEST" "set_b"
   [ ("int_b0" ,    [t| Int32  |])
   , ("may_str_b1" ,    [t| Maybe String |])
   , ("str_b2" ,    [t| String |]) ]
-  [] [0] $ Just 0)
+  [''Generic] [0] $ Just 0)
 
 
 $(defineTable defaultConfig "TEST" "set_c"
@@ -30,19 +32,19 @@ $(defineTable defaultConfig "TEST" "set_c"
   , ("str_c1" ,    [t| String |])
   , ("int_c2" ,    [t| Int64  |])
   , ("may_str_c3" ,    [t| Maybe String |]) ]
-  [] [0] $ Just 0)
+  [''Generic] [0] $ Just 0)
 
 
 $(defineTable defaultConfig "TEST" "set_i"
   [ ("int_i0" ,    [t| Int32 |])  ]
-  [] [0] $ Just 0)
+  [''Generic] [0] $ Just 0)
 
 data ABC =
   ABC
   { xJustA :: SetA
   , xJustB :: SetB
   , xJustC :: SetC
-  }
+  } deriving Generic
 
 $(makeRelationalRecordDefault ''ABC)
 
@@ -51,7 +53,7 @@ data Abc =
   { yJustA :: SetA
   , yMayB  :: Maybe SetB
   , yMayC  :: Maybe SetC
-  }
+  } deriving Generic
 
 $(makeRelationalRecordDefault ''Abc)
 
