@@ -1,9 +1,10 @@
 
 import Test.QuickCheck.Simple (Test, defaultMain)
+import qualified Test.QuickCheck.Simple as QSimple
 
 import Control.Applicative ((<$>), (<*>))
 
-import Lex (eqProp)
+import Lex (eqProp, eqProp')
 import Model
 
 import Data.Int (Int32, Int64)
@@ -77,12 +78,31 @@ assignX = derivedUpdate $ \_proj -> do
   intA0' <-# value (0 :: Int32)
   return unitPlaceHolder
 
-registerX :: Insert ()
+registerX :: Insert (String, Maybe String)
 registerX = derivedInsertValue $ do
-  intA0' <-# value 1
-  strA1' <-# value "Hello"
-  strA2' <-# value "World"
-  return unitPlaceHolder
+  intC0' <-# value 1
+  (ph1, ()) <- placeholder (\ph' -> strC1' <-# ph')
+  intC2' <-# value 2
+  (ph2, ()) <- placeholder (\ph' -> mayStrC3' <-# ph')
+  return $ ph1 >< ph2
+
+eqChunkedInsert :: String
+                -> Insert a
+                -> String
+                -> String
+                -> Test
+eqChunkedInsert name ins prefix row =
+    maybe
+    (name, success)
+    (\(sql, n) ->
+      let estimate =
+            unwords
+            $ prefix
+            : replicate (n - 1) (row ++ ",") ++ [row]
+      in eqProp' name id sql estimate)
+    $ chunkedInsert ins
+  where
+    success = QSimple.Bool Nothing True
 
 monadic :: [Test]
 monadic =
@@ -111,7 +131,9 @@ monadic =
   , eqProp "update"      assignX
     "UPDATE TEST.set_a SET int_a0 = 0"
   , eqProp "insert"      registerX
-    "INSERT INTO TEST.set_a (int_a0, str_a1, str_a2) VALUES (1, 'Hello', 'World')"
+    "INSERT INTO TEST.set_c (int_c0, str_c1, int_c2, may_str_c3) VALUES (1, ?, 2, ?)"
+  , eqChunkedInsert "insert chunked" registerX
+    "INSERT INTO TEST.set_c (int_c0, str_c1, int_c2, may_str_c3) VALUES" "(1, ?, 2, ?)"
   ]
 
 _p_monadic :: IO ()
@@ -484,7 +506,18 @@ ordFlatX =  relation $ do
   on $ just (a ! strA2') .=. b ?! strB2'
 
   orderBy (a ! strA1') Asc
-  orderBy (b ?! mayStrB1') Desc
+  orderBy' (b ?! mayStrB1') Desc NullsLast
+
+  return $ (,) |$| a |*| b
+
+ordFlatY :: Relation () (SetA, Maybe SetB)
+ordFlatY =  relation $ do
+  a <- query setA
+  b <- queryMaybe setB
+  on $ just (a ! strA2') .=. b ?! strB2'
+
+  orderBy (a ! strA1') Asc
+  orderBy' b Desc NullsLast
 
   return $ (,) |$| a |*| b
 
@@ -494,7 +527,7 @@ ordAggX =  aggregateRelation $ do
 
   gc1 <- groupBy $ c ! strC1'
 
-  orderBy (sum' $ c ! intC0') Asc
+  orderBy' (sum' $ c ! intC0') Asc NullsFirst
 
   return $ gc1 >< count (c ! intC0')
 
@@ -507,10 +540,16 @@ orders =
     "SELECT ALL T0.int_a0 AS f0, T0.str_a1 AS f1, T0.str_a2 AS f2, \
     \           T1.int_b0 AS f3, T1.may_str_b1 AS f4, T1.str_b2 AS f5 \
     \  FROM TEST.set_a T0 LEFT JOIN TEST.set_b T1 ON (T0.str_a2 = T1.str_b2) \
-    \  ORDER BY T0.str_a1 ASC, T1.may_str_b1 DESC"
+    \  ORDER BY T0.str_a1 ASC, T1.may_str_b1 DESC NULLS LAST"
+  , eqProp "order-by - flag 2" ordFlatY
+    "SELECT ALL T0.int_a0 AS f0, T0.str_a1 AS f1, T0.str_a2 AS f2, \
+    \           T1.int_b0 AS f3, T1.may_str_b1 AS f4, T1.str_b2 AS f5 \
+    \  FROM TEST.set_a T0 LEFT JOIN TEST.set_b T1 ON (T0.str_a2 = T1.str_b2) \
+    \  ORDER BY T0.str_a1 ASC, T1.int_b0 DESC NULLS LAST, \
+    \           T1.may_str_b1 DESC NULLS LAST, T1.str_b2 DESC NULLS LAST"
   , eqProp "order-by - aggregated" ordAggX
     "SELECT ALL T0.str_c1 AS f0, COUNT(T0.int_c0) AS f1 \
-    \  FROM TEST.set_c T0 GROUP BY T0.str_c1 ORDER BY SUM(T0.int_c0) ASC"
+    \  FROM TEST.set_c T0 GROUP BY T0.str_c1 ORDER BY SUM(T0.int_c0) ASC NULLS FIRST"
   ]
 
 partitionX :: Relation () (String, Int64)
@@ -608,8 +647,12 @@ effs :: [Test]
 effs =
   [ eqProp "insert" insertX
     "INSERT INTO TEST.set_a (int_a0, str_a1, str_a2) VALUES (?, ?, ?)"
+  , eqChunkedInsert "insert chunked" insertX
+    "INSERT INTO TEST.set_a (int_a0, str_a1, str_a2) VALUES" "(?, ?, ?)"
   , eqProp "insert1" insertI
     "INSERT INTO TEST.set_i (int_i0) VALUES (?)"
+  , eqChunkedInsert "insert1 chunked" insertI
+    "INSERT INTO TEST.set_i (int_i0) VALUES" "(?)"
   , eqProp "insertQuery" insertQueryX
     "INSERT INTO TEST.set_b (int_b0, str_b2, str_b2) SELECT int_a0, str_a1, str_a2 FROM TEST.set_a"
   , eqProp "updateKey" updateKeyX
